import numpy as np
from scipy import optimize
from skimage import draw
import cv2 

def crop_img(img, crop):
    # crop is: ymin, ymax, xmin, xmax
    ymin, ymax, xmin, xmax = crop
    img = img[ymin:ymax, xmin:xmax, :]
    y, x, _ = img.shape
    delta = x - y
    if delta > 0:
        _padding = [(abs(delta), 0), (0, 0), (0, 0)]
        img = np.pad(img, _padding)
    elif delta < 0:
        _padding = [(0, 0), (abs(delta), 0), (0, 0)]
        img = np.pad(img, _padding)

    return img

class Cropper:
    def __init__(self):
        pass

    def __call__(self, img, crop=None, scale=None, scale_method=cv2.INTER_AREA):
        """
        img: numpy array of image
        crop: [y_min, y_max, x_min, x_max]
        scale: [width, height]
        """

        if crop is not None:
            img = crop_img(img, crop)
        else:
            raise Exception("Automatic crop detection not implemented yet")

        if scale is not None:
            img = cv2.resize(img, dsize=scale, interpolation=scale_method)

        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)

        return np.array(img)



# agl_visualization/run_visualizer.py

import subprocess
import argparse

def main():
    parser = argparse.ArgumentParser(description="Visualize Predictions")
    parser.add_argument("--file", type=str, help="Path to JSON file containing predictions", required=True)
    args = parser.parse_args()

    # Replace this with the path where prediction_visualizer.py resides
    streamlit_script_path = "agl_predict_endo_frame/prediction_visualizer.py"
    
    # Using subprocess to run the Streamlit command
    subprocess.run(f'streamlit run {streamlit_script_path} -- --file {args.file}', shell=True)

if __name__ == "__main__":
    main()




from .inference_dataset import InferenceDataset
from torch.utils.data import DataLoader
import torch
from torch import nn
import json

sample_config = {
    # mean and std for normalization
    "mean": (0.45211223, 0.27139644, 0.19264949),
    "std": (0.31418097, 0.21088019, 0.16059452),
    # Image Size
    "size_x": 716,
    "size_y": 716,
    # how to wrangle axes of the image before putting them in the network
    "axes": [2,1,0],
    "batchsize": 16,
    "num_workers": 1, # always 1 for Windows systems
    # maybe add sigmoid after prediction?
    "activation": nn.Sigmoid(),
    "labels": ['appendix',  'blood',  'diverticule',  'grasper',  'ileocaecalvalve',  'ileum',  'low_quality',  'nbi',  'needle',  'outside',  'polyp',  'snare',  'water_jet',  'wound']
}

class Classifier():
    def __init__(self, model=None, config=sample_config, verbose = False):
        self.config = config
        self.model = model
        self.verbose = verbose

    def pipe(self, paths, crops, verbose = None):
        if verbose is None:
            verbose = self.verbose

        dataset = InferenceDataset(paths, crops, self.config)
        if verbose:
            print("Dataset created")

        dl = DataLoader(
            dataset=dataset,
            batch_size=self.config["batchsize"],
            num_workers=self.config["batchsize"],
            shuffle = False,
            pin_memory=True
        )
        if verbose:
            print("Dataloader created")

        predictions = []

        with torch.inference_mode():
            if self.verbose:
                print("Starting inference")
            for i,batch in enumerate(dl):
                prediction = self.model(batch.cuda())
                prediction = self.config["activation"](prediction).cpu().tolist()#.numpy().tolist()
                predictions += prediction
                if self.verbose and i==0:
                    print("First batch done")

        return predictions

    def __call__(self, image, crop=None):
        return self.pipe([image], [crop])

    def readable(self, predictions):
        return {label: prediction for label, prediction in zip(self.config["labels"], predictions)}
    
    def get_prediction_dict(self, predictions, paths):
        json_dict = {
            "labels": self.config["labels"],
            "paths": paths,
            "predictions": predictions
        }
        
        return json_dict
    
    def get_prediction_json(self, predictions, paths, json_target_path: str = None):
        if not json_target_path:
            json_target_path = "predictions.json"

        json_dict = self.get_prediction_dict(predictions, paths)

        with open(json_target_path, 'w') as f:
            json.dump(json_dict, f)
        
        if self.verbose:
            print(f"Saved predictions to {json_target_path}")




import streamlit as st
import matplotlib.pyplot as plt
import json
from PIL import Image



# Load the JSON file
@st.cache_data
def load_data(file_path):
    with open(file_path, 'r') as f:
        data = json.load(f)
    return data

# Streamlit App
def main(file_path):
    st.title("Visualize Predictions")

    # Load the data
    data = load_data(file_path)

    # Dropdown to select label
    selected_label = st.selectbox('Select Label', options=data['labels'])

    # Get the index of the selected label
    label_index = data['labels'].index(selected_label)

    # Extract predictions for the selected label
    selected_predictions = [pred[label_index] for pred in data['predictions']]

    # Line plot for the selected label's prediction values
    st.subheader(f"Line plot for {selected_label}")

    fig, ax = plt.subplots(figsize=(10, 6))  # Explicitly create a figure and axis
    ax.plot(selected_predictions)
    ax.set_xlabel('Frame')
    ax.set_ylabel('Prediction Value')
    ax.set_title(f'Predictions for {selected_label}')

    st.pyplot(fig)  # Pass the figure object explicitly


    # Slider to select frame
    frame_idx = st.slider('Select a frame', 0, len(data['paths']) - 1)

    # Display the frame
    image_path = data['paths'][frame_idx]
    image = Image.open(image_path)
    st.image(image, caption=f"Frame {frame_idx}", use_column_width=True)

if __name__ == '__main__':
    main(args.file)



from torch.utils.data import Dataset
import cv2
import torch
import numpy as np
import warnings
from .preprocess import Cropper
from PIL import Image
import albumentations as A

class InferenceDataset(Dataset):
    def __init__(self, paths, crops, config):
        self.paths = paths
        self.crops = crops
        self.cropper = Cropper()
        self.config = config

        self.img_transforms = A.Compose([
            A.Normalize(
                mean=self.config["mean"],
                std=self.config["std"],
            max_pixel_value=255)
        ])
        

    def __len__(self):
        return len(self.paths)

    def __getitem__(self, idx):
        image = cv2.imread(self.paths[idx])
                
        crop = self.crops[idx]

        cropped = self.cropper(
            image, 
            crop, 
            scale=[
                self.config["size_x"],
                self.config["size_y"]
            ]    
        )
        
        img = self.img_transforms(image=cropped)["image"]
        img = torch.tensor(img, dtype = torch.float32)
        img = img.permute(self.config["axes"])

        return img
